unit Protohdr;
{
  PROTOHDR.PAS

  Original source is PROTOHDR.H
  Written by Sang-Eun Han
  (seh@brabo1.korea.ac.kr,  http://widecomm.korea.ac.kr/~seh).

  Convert to Delphi 4.0 by: Jagad (don@indo.net.id)
  May 10, 1999: Make faster TOULONG and TOUSHORT function and make it works
                with D3 by: Francois Piette (francois.piette@pophost.eunet.be)
}

{$ALIGN ON}

interface

type
// Ethernet Frame Header
(*
typedef	struct	_ETHERNET_HDR {
	UCHAR	Destination[6];
	UCHAR	Source[6];
	UCHAR	Protocol[2];
	UCHAR	Data[1];
} ETHERNET_HDR, *PETHERNET_HDR;
*)
ETHERNET_HDR = packed record
  Destination: array[0..5] of Byte;
  Source:      array[0..5] of Byte;
  Protocol:    array[0..1] of Byte;
  Data:        array[0..0] of Byte;
end;
PETHERNET_HDR = ^ETHERNET_HDR;

const
//rfc1340
PROTO_IP      =	$0800;
PROTO_ARP     =	$0806;
PROTO_XNS     =	$0600;
PROTO_SNMP    =	$814C;
PROTO_OLD_IPX =	$8137;
PROTO_NOVELL  =	$8138;
PROTO_IPNG    =	$86DD;


OFFSET_IP =	14;

type
// IPv4 Header
(*
typedef	struct	_IP_RHDR {
	UCHAR	VerLen;
	UCHAR	Service;
	UCHAR	Length[2];
	UCHAR	Ident[2];
	UCHAR	Flagoff[2];
	UCHAR	Timelive;
	UCHAR	Protocol;
	UCHAR	Checksum[2];
	UCHAR	Source[4];
	UCHAR	Destination[4];
	UCHAR	Data[1];
} IP_RHDR, *PIP_RHDR;
*)
IP_RHDR = packed record
  Verlen:       Byte;
  Service:      Byte;
  Length:       array[0..1] of Byte;
  Ident:        array[0..1] of Byte;
  Flagoff:      array[0..1] of Byte;
  TimeLive:     byte;
  Protocol:     byte;
  Checksum:     array[0..1] of byte;
  Source:       array[0..3] of byte;
  Destination:  array[0..3] of byte;
  Data:         array[0..0] of byte;
end;
PIP_RHDR = ^IP_RHDR;

// IPv6 Header
(*
typedef	struct	_IPNG_RHDR {
	byte	VerPrio;
	byte	FlowLabel[3];
	byte	Length[2];
	byte	NextHdr;
	byte	HopLimit;
	byte	Source[16];
	byte	Destination[16];
	byte	Data[1];
} IPNG_RHDR, *PIPNG_RHDR;
*)
IPNG_RHDR = packed record
  VerPrio:        byte;
  FlowLabel:      array[0..2] of byte;
  Length:         array[0..1] of byte;
  NextHadr:       byte;
  HopLimit:       byte;
  Source:         array[0..15] of byte;
  Destination:    array[0..15] of byte;
  Data:           array[0..0] of byte;
end;
PIPNG_RHDR = ^IPNG_RHDR;

// TCP Header, RFC793
(*
typedef	struct	_TCP_RHDR {
	byte	Source[2];
	byte	Destination[2];
	byte	Seq[4];
	byte	Ack[4];
	byte	Rsvd0:4;
	byte	Offset:4;
	byte	Flags:6;
	byte	Rsvd1:2;
	byte	Window[2];
	byte	Checksum[2];
	byte	UrgPoint[2];
	byte	Data[1];
} TCP_RHDR, *PTCP_RHDR;
*)
TCP_RHDR = packed record
  Source:        array[0..1] of byte; //Source Port
  Destination:   array[0..1] of byte; //Destination Port
  Seq:           array[0..3] of byte;
  Ack:           array[0..3] of byte;
  Rsvd0_Off:     byte;
  Flags_Rsvd1:   byte;
  Window:        array[0..1] of byte;
  Checksum:      array[0..1] of byte;
  UrgPoint:      array[0..1] of byte;
  Data:          array[0..0] of byte;
end;
PTCP_RHDR = ^TCP_RHDR;

const
TCP_FLAG_FIN =	$01;
TCP_FLAG_SYN =	$02;
TCP_FLAG_RST =	$04;
TCP_FLAG_PSH =	$08;
TCP_FLAG_ACK =	$10;
TCP_FLAG_URG =	$20;

PROTO_TCP = 6;

type
// UDP Header
(*
typedef	struct	_UDP_RHDR {
	byte	Source[2];
	byte	Destination[2];
	byte	Length[2];
	byte	Checksum[2];
	byte	Data[1];
} UDP_RHDR, *PUDP_RHDR;
*)
UDP_RHDR = packed record
  Source:        array[0..1] of byte;
  Destination:   array[0..1] of byte;
  Length:        array[0..1] of byte;
  Checksum:      array[0..1] of byte;
  Data:          array[0..0] of byte;
end;
PUDP_RHDR = ^UDP_RHDR;


// Pseudo Header for evaluating TCP/UDP Checksum
(*
typedef	struct	_PSU_RHDR {
	byte	Source[4];
	byte	Destination[4];
	byte	Zero;
	byte	Protocol;
	byte	Length[2];
	byte	Data[1];
} PSU_RHDR, *PPSU_RHDR;
*)
PSU_RHDR = packed record
  Source:        array[0..3] of byte;
  Destination:   array[0..3] of byte;
  Zero:          byte;
  Protocol:      byte;
  Length:        array[0..1] of byte;
  Data:          array[0..0] of byte;
end;
PPSU_RHDR = ^PSU_RHDR;


//Borland CBuilder is 32 bit Win App
//#ifdef	WIN32
//#define	TOUSHORT(x)	(USmallint)(*(x)<<8|*(x+1))
//#define	TOULONG(x)	(ULONG)(*(x)<<24|*(x+1)<<16|*(x+2)<<8|*(x+3))
//#else
//#define	TOUSmallint(x)	(*(USmallint *)x)	//Big-Endian
//#define	TOULONG(x)	(*(ULONG *)x)
//#endif

function TOUSmallint(x: PChar): Smallint;
function TOULONG(x: PChar): LongWord;

implementation

function TOUSmallint(x: PChar): Smallint;
begin
    Result := (Smallint(x^) shl 8) or (Smallint((x + 1)^));  // FP May 10, 1999
end;

function TOULONG(x: PChar): LongWord;
begin
    Result := (LONGWORD(x^) shl 24) or                    // FP May 10, 1999
              (LONGWORD((x + 1)^) shl 16) or              // FP May 10, 1999
              (LONGWORD((x + 2)^) shl 8) or               // FP May 10, 1999
              (LONGWORD((x + 3)^));                       // FP May 10, 1999
end;

end.
