{
********************************************************************************
--------------------------------------------------------------------------------
                                    TZNIFFER

                for Packet Capture Driver by Politecnico di Torino

                         Written by Lars Peter Christiansen
--------------------------------------------------------------------------------
 TERMS AND CONDITIONS OF USE.
 All of this software is Copyright(C) 2002 Lars Peter Christiansen.

 The author of this software assumes no liability for damages caused under
 any circumstances whatsoever, and is under no obligation. Use of the software
 indicates acceptance of all conditions contained in this document. If you do
 not agree to these terms, you must delete this software immediately.

 You may distribute the archive in which this software is distributed, but
 under no circumstances must this archive be changed. Distributing a modified
 archive is a violation of the software license.

 If you do redistribute this software, please let me know at the email address
 given below.

 If you have any questions, requests, bug reports, etc., please contact me at
 the address given below.

Lars Peter Christiansen
Email  : lp@nzlab.dk
Website: http://www.nzlab.dk

--------------------------------------------------------------------------------

                          [ user application ]
                          [     TZniffer     ] <- you are here!
                          [       PCAP       ]
                          [    Netadapter    ]


********************************************************************************
}
unit Zniffer;

interface
uses Windows,
     Classes,
     Sysutils,
     Pcap,
     Packet32;
Type

PETHERNET_HDR = ^ETHERNET_HDR;
ETHERNET_HDR = packed record
  Destination: array[0..5] of UCHAR;
  Source:      array[0..5] of UCHAR;
  Protocol:    array[0..1] of UCHAR;
  Data:        array[0..0] of UCHAR;
end;

PIP_RHDR = ^IP_RHDR;
IP_RHDR = packed record
  Verlen:       UCHAR;  //4bit version 4bit length (bytes/8)
  Service:      UCHAR;  // TOS
  Length:       WORD;
  Ident:        WORD;
  Flagoff:      array[0..1] of UCHAR; //3bit flag - 13 bit offset
  TimeLive:     UCHAR;
  Protocol:     UCHAR;
  Checksum:     WORD;
  SrcIP:        array[0..3] of UCHAR;
  DestIP:       array[0..3] of UCHAR;
  Data:         array[0..0] of UCHAR;
end;

// Added ( Lars Peter Christiansen 13-04-2001)
PTCP_RHDR = ^TCP_RHDR;
TCP_RHDR = Packed record
  SrcPort     : WORD;
  DestPort    : WORD;
  SequenceNr  : array[0..3] of UCHAR;
  AckNumber   : array[0..3] of UCHAR;
  LenResvFlags: array[0..1] of UCHAR; //length(4bits) rsvd(6bits) flags(6bits)
  WindowSize  : array[0..1] of UCHAR;
  Checksum    : array[0..1] of UCHAR;
  UrgentPtr   : array[0..1] of UCHAR;
  Data        : array[0..0] of UCHAR;
end;

  Tzniffer = class;

  // Thread that listens to selected Netadapter
  TZnifferThread = Class(Tthread)
  private
    Z         : Tzniffer;
  public
    ReadTimes : integer;
    Constructor Create(Zniffer:Tzniffer);
    Destructor Destroy;override;
    Procedure Execute;override;
  end;

  // The main TZniffer Class here
  TZniffer = Class
  private
    FPCAP : PPCAP;                          // Handle to the pcapdriver
    Fadapters :TstringList;                 // the adapters found on the system
    FadapterIndex:Integer;                  // current adapter
    FThread : TZnifferThread;               // The listening thread
    Fsnooping : Boolean;                    // Flag indicating snooping activity
    Function GetAdapters(Var ErrStr:string) : boolean;
    procedure ThreadTerminate(Sender:tobject);
    procedure SetAdapterIndex(const Value: integer);
  public
    OnPacket : Procedure(Data:pointer;recvbytes:Word) of Object;
    Constructor Create;
    Destructor Destroy;override;
    Function Activate(var ErrStr:string) : boolean;
    Function Deactivate(var ErrStr:string):boolean;
    property Snooping : boolean Read Fsnooping;
    property Adapters : TstringList read Fadapters;
    property AdapterIndex:integer read FadapterIndex write SetAdapterIndex;
  end;

Const //Look in rfc1340 dokument
      PROTO_IP      =	$0800;

function TOUSHORT(x: PChar): SHORT;  // Ripped C Makro
implementation

function TOUSHORT(x: PChar): SHORT;
begin Result := (SHORT(x^) shl 8) or (SHORT((x + 1)^));  // FP May 10, 1999
end;


{ TZniffer }
constructor TZniffer.Create;
var S:string;
begin
  FAdapters := Tstringlist.Create;
  FadapterIndex := 0;
  FPCAP := Nil;
  Fsnooping:=false;
  GetAdapters(s);
end;

destructor TZniffer.Destroy;
Var E:string;
begin

  DeActivate(E);
  Fadapters.free;
  Fadapters := nil;

  inherited;

end;



//------------------------------------------------------------------------------
//  ACTIVATE SNOOP AND START READTHREAD
//------------------------------------------------------------------------------
function TZniffer.Activate(var ErrStr: string): boolean;
begin
Result := false;

  // Check if Snooping is active
  if Fsnooping or (FPCAP<>nil) then
  begin
    ErrStr := 'Snooping already activated';
    exit;
  end;

  // Open Driver and NetAdapter
  FPCAP := Pcap.pcap_open_live(Pchar(FAdapters[FAdapterindex]),
                            DEFAULT_SNAPLEN,TRUE,100,ErrStr);
  if FPCAP = nil then exit;

  if not Assigned(OnPacket) then
    begin
      ErrStr:='No Packet Read Callback function assigned';
      exit;
    end;

  // Start Snoop Read Thread
  FThread := TZnifferThread.create(self);
  Fthread.ReadTimes := 10;
  Fthread.OnTerminate := ThreadTerminate;
  Fthread.FreeOnTerminate := false;
  Fthread.resume;
  FSnooping := True;

result:=true;
end;


function TZniffer.Deactivate(var ErrStr: string): boolean;
var
  P:Tpacket;
  T:longword;
begin
result := false;
  if (not Fsnooping) then begin errstr:='Snooping not active';exit;end;
  if FThread=nil then begin errstr:='No thread to stop';exit;end;

  // Stop Snooping Thread
  FThread.Terminate;
  FThread.WaitFor;
  FThread.Free;
  Fthread := nil;

  // Release Driver Handle
  Pcap_Close(FPCAP);

result :=true;
end;


//------------------------------------------------------------------------------
//  GET ADAPTERS IN SYSTEM
//------------------------------------------------------------------------------
function TZniffer.GetAdapters(var ErrStr:string): boolean;
begin
result:=false;

  if FAdapters=nil then
    begin
      ErrStr :='Memory for Adapterlist not allocated';
      exit;
    end;

  Fadapters.commatext := Pcap.pcap_GetAdapternames(',',ErrStr);
  if Fadapters.CommaText='' then exit;

  result := true;
end;

//------------------------------------------------------------------------------
//  { TZnifferThread }
//
//  And Snoop Driver Callback function which cannot be procedure of object!
//------------------------------------------------------------------------------

procedure CaptureCB(User:pointer;const Header:Ppcap_pkthdr;const Data:pchar);
begin
    TZniffer(user).OnPacket(Data,Header.len);
end;

constructor TZnifferThread.Create(Zniffer:Tzniffer);
begin
  ReadTimes:=0;
  Z:=Zniffer;
  PacketSetReadTimeout(z.Fpcap.Adapter,100);
  inherited Create(TRUE);
end;

destructor TZnifferThread.Destroy;
begin
  inherited;

end;

procedure TZnifferThread.Execute;
begin
  if Z=nil then exit;

  While Not Terminated do
    begin
      Pcap_Read(Z.FPCAP,0,CaptureCB,Pointer(Z));
    end;

end;


procedure TZniffer.ThreadTerminate(Sender: tobject);
begin
  Fsnooping:=false;
end;

procedure TZniffer.SetAdapterIndex(const Value: integer);
begin
  if (value>-1) and (value<Adapters.count) then
    FadapterIndex := Value;
end;

end.
